/*
 * Copyright 2012 Giorgio Vazzana
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Compile: gcc -Wall -O montecarlo_pi.c -o montecarlo_pi */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>

/*
  Area_circle = pi·r²
  Area_square = (2·r)²
  Ratio: q = pi/4
*/

#define BITS 24

uint16_t read_uint16(int *reached_eof)
{
	int i, c;
	uint16_t value = 0;

	for (i = 0; i < 16; i++) {
		c = fgetc(stdin);
		c = (c == '0') ? 0 : 1;
		value = (value << 1) | c;
		c = fgetc(stdin); /* \n */
		if (c == EOF) {
			*reached_eof = 1;
			break;
		}
	}
	return value;
}

uint32_t read_uint24(int *reached_eof)
{
	int i, c;
	uint32_t value = 0;

	for (i = 0; i < 24; i++) {
		c = fgetc(stdin);
		c = (c == '0') ? 0 : 1;
		value = (value << 1) | c;
		c = fgetc(stdin); /* \n */
		if (c == EOF) {
			*reached_eof = 1;
			break;
		}
	}
	return value;
}

uint32_t read_uint32(int *reached_eof)
{
	int i, c;
	uint32_t value = 0;

	for (i = 0; i < 32; i++) {
		c = fgetc(stdin);
		c = (c == '0') ? 0 : 1;
		value = (value << 1) | c;
		c = fgetc(stdin); /* \n */
		if (c == EOF) {
			*reached_eof = 1;
			break;
		}
	}
	return value;
}


int main(int argc, char *argv[])
{
	int n, maxpoints, inner;
	int reached_eof = 0;
	double x, y, pi;

	if (argc < 2) {
		printf("Usage: cat seq | montecarlo_pi maxpoints\n");
		return 1;
	}

	maxpoints = atoi(argv[1]);
	inner = 0;
	for (n = 1; n <= maxpoints; n++) {
#if BITS==16
		x = (double) read_uint16(&reached_eof) / (double) UINT16_MAX;
		y = (double) read_uint16(&reached_eof) / (double) UINT16_MAX;
#elif BITS==24
		x = (double) read_uint24(&reached_eof) / (double) ((1U << 24) -1);
		y = (double) read_uint24(&reached_eof) / (double) ((1U << 24) -1);
#elif BITS==32
		x = (double) read_uint32(&reached_eof) / (double) UINT32_MAX;
		y = (double) read_uint32(&reached_eof) / (double) UINT32_MAX;
#else
		x = (double)rand() / (double) RAND_MAX;
		y = (double)rand() / (double) RAND_MAX;
#endif

		if (reached_eof)
			break;

		if (x*x + y*y <= 1.0)
			inner++;

		pi = 4.0 * (double) inner / (double) n;

		fprintf(stderr, "%.10f %.10f\n", x, y);
		fprintf(stdout, "n=%5d  pi=%.9f  e=% .3f%%\n", n, pi, (pi / M_PI - 1.0) * 100.0);
//		fprintf(stderr, "%5d  %.9f  % .3f\n", n, pi, (pi / M_PI - 1.0) * 100.0);
	}

	return 0;
}
